unit EloTextResIni;

interface

uses
  SysUtils, Classes, IniFiles, Controls, Forms, TypInfo, Dialogs, Menus;

const
     AllClasses: String = 'All classes';

type
  TActionText = (atRead, atWrite);
  TOnBeforeIni = procedure(ControlName: String; var CanExecute: Boolean) of object;
  TOnAfterExecute = procedure(Sender: TObject; Result: Boolean) of object;

  TEloTextResIni = class(TComponent)
  private
    { Private declarations }
    FActionText: TActionText;
    FFormControl: TCustomForm;
    FFileName: TFileName;
    FListClassControl: TStringList;
    FListPropertyControl: TStringList;
    FListSkipName: TStringList;
    FUseForForms: Boolean;
    FOnBeforeExecute: TNotifyEvent;
    FOnAfterExecute: TOnAfterExecute;
    FOnBeforeRead: TOnBeforeIni;
    FOnBeforeWrite: TOnBeforeIni;

    function GeTCustomFormControl: TCustomForm;
    procedure SetCustomFormControl(Value: TCustomForm);
    procedure SetFileName(Value: TFileName);
    procedure SetListClassControl(const Value: TStringList);
    procedure SetListPropertyControl(const Value: TStringList);
    procedure SetListSkipName(const Value: TStringList);
    procedure SetUseForForms(Value: Boolean);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function CheckSkipName(aName: String): Boolean; virtual;
    function Execute: Boolean;
    function ReadResources: Boolean; virtual;
    function WriteResources: Boolean; virtual;
  published
    { Published declarations }
    property ActionText: TActionText read FActionText write FActionText default atRead;
    property FormControl: TCustomForm read GeTCustomFormControl write SeTCustomFormControl;
    property FileName: TFileName read FFileName write SetFileName;
    property ListClassControl: TStringList read FListClassControl write SetListClassControl;
    property ListPropertyControl: TStringList read FListPropertyControl write SetListPropertyControl;
    property ListSkipName: TStringList read FListSkipName write SetListSkipName;
    property UseForForms: Boolean read FUseForForms write SetUseForForms default False;
    property OnBeforeExecute: TNotifyEvent read FOnBeforeExecute write FOnBeforeExecute;
    property OnAfterExecute: TOnAfterExecute read FOnAfterExecute write FOnAfterExecute;
    property OnBeforeRead: TOnBeforeIni read FOnBeforeRead write FOnBeforeRead;
    property OnBeforeWrite: TOnBeforeIni read FOnBeforeWrite write FOnBeforeWrite;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('EloSoft', [TEloTextResIni]);
end;

constructor TEloTextResIni.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FListClassControl:= TStringList.Create;
  FListClassControl.Add(AllClasses);
  FListPropertyControl:= TStringList.Create;
  FListPropertyControl.Add('Caption');
  FListPropertyControl.Add('Text');
  FListPropertyControl.Add('Hint');
  FListSkipName:= TStringList.Create;
  FUseForForms:= False;
  FActionText:= atRead;
  if (AOwner<> nil)
  and (AOwner is TCustomForm) then FFormControl:= (AOwner as TCustomForm)
  else FFormControl:= nil;
end;

destructor TEloTextResIni.Destroy;
begin
  FFormControl:= nil;
  FListClassControl.Free;
  FListPropertyControl.Free;
  FListSkipName.Free;
  inherited Destroy;
end;

function TEloTextResIni.GeTCustomFormControl: TCustomForm;
begin
  Result:= FFormControl;
end;

procedure TEloTextResIni.SetCustomFormControl(Value: TCustomForm);
begin
  if Value<> FFormControl then FFormControl:= Value;
end;

procedure TEloTextResIni.SetFileName(Value: TFileName);
begin
  if Value<> FFileName then FFileName:= Value;
end;

procedure TEloTextResIni.SetListClassControl(const Value: TStringList);
begin
  if Assigned(FListClassControl) then FListClassControl.Assign(Value)
  else FListClassControl:= Value;
end;

procedure TEloTextResIni.SetListPropertyControl(const Value: TStringList);
begin
  if Assigned(FListPropertyControl) then FListPropertyControl.Assign(Value)
  else FListPropertyControl:= Value;
end;

procedure TEloTextResIni.SetListSkipName(const Value: TStringList);
begin
  if Assigned(FListSkipName) then FListSkipName.Assign(Value)
  else FListSkipName:= Value;
end;

procedure TEloTextResIni.SetUseForForms(Value: Boolean);
begin
  if Value<> FUseForForms then FUseForForms:= Value;
end;

function TEloTextResIni.CheckSkipName(aName: String): Boolean;
var
   i: Integer;
begin
  Result:= False;
  if FListSkipName.Count<= 0 then exit;
  for i:= 0 to FListSkipName.Count- 1 do
      if (aName= FListSkipName.Strings[i])
      or ((pos('**', FListSkipName.Strings[i])> 0)
         and (pos(copy(FListSkipName.Strings[i], 1, length(FListSkipName.Strings[i])- 2), aName)> 0)) then
      begin
       Result:= True;
       exit;
      end;
end;

function TEloTextResIni.Execute: Boolean;
begin
  Result:= False;
  if (FFormControl= nil)
  or (FListClassControl.Count< 1)
  or (FListPropertyControl.Count< 1) then exit;
  if Assigned(FOnBeforeExecute) then FOnBeforeExecute(Self);
  if FActionText= atRead then Result:= ReadResources
  else if FActionText= atWrite then Result:= WriteResources;
  if Assigned(FOnAfterExecute) then FOnAfterExecute(Self, Result);
end;

function TEloTextResIni.ReadResources: Boolean;
var
   i, k: Integer;
   resFile: TIniFile;
   SectionName, aPropertyName, aPropertyValue: String;
   CanWrite: Boolean;
   clType: TClass;

   function CheckListClassControl(aclType: TClass): Boolean;
   var
      n: Integer;
   begin
     Result:= True;
     if (AnsiUpperCase(FListClassControl.Strings[0])=
        AnsiUpperCase(AllClasses)) then exit;
     for n:= 0 to FListClassControl.Count- 1 do
         if FListClassControl.Strings[n]= aclType.ClassName then exit
         else if aclType.ClassParent<> nil then
                 if CheckListClassControl(aclType.ClassParent) then exit;
     Result:= False;
   end;

   function CheckPropertyValue(s: String): String;
   begin
     if pos('<"13">', s)> 0 then s:= StringReplace(s, '<"13">', #13, [rfReplaceAll, rfIgnoreCase]);
     if pos('<"10">', s)> 0 then s:= StringReplace(s, '<"10">', #10, [rfReplaceAll, rfIgnoreCase]);
     Result:= s;
   end;

begin
  if not FileExists(FFileName) then
  begin
   ShowMessage(SysErrorMessage(GetLastError));
   Result:= False;
   exit;
  end;
  
  clType:= nil;
  resFile:=TIniFile.Create(FFileName);
  try
   try
    SectionName:= FFormControl.Name;
    if FUseForForms then
    begin
     FFormControl.Caption:= resFile.ReadString(
                              SectionName,
                              FFormControl.Name+ '_Caption',
                              FFormControl.Caption);
     FFormControl.Hint:= resFile.ReadString(
                           SectionName,
                           FFormControl.Name+ '_Hint',
                           FFormControl.Hint);
    end;

    for i:= 0 to FFormControl.ComponentCount- 1 do
    begin
     if not CheckListClassControl(FFormControl.Components[i].ClassType)
     or CheckSkipName(FFormControl.Components[i].Name) then continue
     else clType:= FFormControl.Components[i].ClassType;
     if clType= nil then continue;
     CanWrite:= True;
     if Assigned(OnBeforeWrite) then FOnBeforeWrite(FFormControl.Components[i].Name, CanWrite);
     if not CanWrite then continue;
     for k:= 0 to FListPropertyControl.Count- 1 do
     begin
      aPropertyName:= FListPropertyControl.Strings[k];
      aPropertyValue:= '';
      if GetPropInfo((FFormControl.Components[i] as clType), aPropertyName)<> nil then
      begin
       aPropertyValue:= CheckPropertyValue(
                          resFile.ReadString(
                            SectionName,
                            FFormControl.Components[i].ClassName+ '_'+
                            FFormControl.Components[i].Name+ '_'+
                            aPropertyName,
                            aPropertyValue));
       SetStrProp(FFormControl.Components[i], aPropertyName, aPropertyValue);
      end; // end if GetPropInfo
     end; // end for k:= 0 to FListPropertyControl.Count- 1
    end; // end for i:= 0 to FFormControl.ComponentCount- 1
    Result:= True;
   except
    Result:= False;
   end;
  finally
   resFile.Free;
  end; // end try
end;

function TEloTextResIni.WriteResources: Boolean;
var
   i, k: Integer;
   resFile: TIniFile;
   SectionName, aPropertyName, aPropertyValue: String;
   CanWrite: Boolean;
   clType: TClass;

   function CheckListClassControl(aclType: TClass): Boolean;
   var
      n: Integer;
   begin
     Result:= True;
     if (AnsiUpperCase(FListClassControl.Strings[0])=
        AnsiUpperCase(AllClasses)) then exit;
     for n:= 0 to FListClassControl.Count- 1 do
         if FListClassControl.Strings[n]= aclType.ClassName then exit
         else if aclType.ClassParent<> nil then
                 if CheckListClassControl(aclType.ClassParent) then exit;
     Result:= False;
   end;

   function CheckPropertyValue(s: String): String;
   begin
     if pos(#13, s)> 0 then s:= StringReplace(s, #13, '<"13">', [rfReplaceAll, rfIgnoreCase]);
     if pos(#10, s)> 0 then s:= StringReplace(s, #10, '<"10">', [rfReplaceAll, rfIgnoreCase]);
     Result:= s;
   end;

begin
  clType:= nil;
  resFile:=TIniFile.Create(FFileName);
  try
   try
    SectionName:= FFormControl.Name;
    resFile.WriteString(
      SectionName,
      FFormControl.Name+ '_Caption',
      FFormControl.Caption);
    resFile.WriteString(
      SectionName,
      FFormControl.Name+ '_Hint',
      FFormControl.Hint);

    for i:= 0 to FFormControl.ComponentCount- 1 do
    begin
     if not CheckListClassControl(FFormControl.Components[i].ClassType)
     or CheckSkipName(FFormControl.Components[i].Name) then continue
     else clType:= FFormControl.Components[i].ClassType;
     if clType= nil then continue;
     CanWrite:= True;
     if Assigned(OnBeforeWrite) then FOnBeforeWrite(FFormControl.Components[i].Name, CanWrite);
     if not CanWrite then continue;
     for k:= 0 to FListPropertyControl.Count- 1 do
     begin
      aPropertyName:= FListPropertyControl.Strings[k];
      aPropertyValue:= '';
      if GetPropInfo((FFormControl.Components[i] as clType), aPropertyName)<> nil then
      begin
       aPropertyValue:= CheckPropertyValue(GetStrProp((FFormControl.Components[i] as clType), aPropertyName));
       resFile.WriteString(
         SectionName,
         FFormControl.Components[i].ClassName+ '_'+
         FFormControl.Components[i].Name+ '_'+
         aPropertyName,
         aPropertyValue);
      end; // end if GetPropInfo
     end; // end for k:= 0 to FListPropertyControl.Count- 1
    end; // end for i:= 0 to FFormControl.ComponentCount- 1
    Result:= True;
   except
    Result:= False;
   end;
  finally
   resFile.Free;
  end; // end try
end;

end.
